"""create_app application factory function and similar things."""
import logging
import os
from logging.config import dictConfig

from flask import Flask, request, send_from_directory

from ._version import get_versions

__version__ = get_versions()['version']
del get_versions


def create_app(instance_path=None, test_config=None):
    app = Flask(__name__, instance_relative_config=True, instance_path=instance_path)

    # if it doesn't already exist, create the instance folder
    os.makedirs(app.instance_path, exist_ok=True)

    # load defaults from config provided with the application
    app.config.from_object('incorporealcms.config.Config')
    # load specific instance configurations
    app.config.from_pyfile('config.py', silent=True)
    if test_config:
        app.config.from_mapping(test_config)

    dictConfig(app.config['LOGGING'])

    logger = logging.getLogger(__name__)

    logger.debug("instance path: %s", app.instance_path)

    @app.before_request
    def log_request():
        logger.info("REQUEST:  %s %s", request.method, request.path)

    @app.after_request
    def log_response(response):
        logger.info("RESPONSE: %s %s: %s", request.method, request.path, response.status)
        return response

    @app.route(f'/{app.config["MEDIA_DIR"]}/<path:filename>')
    def media_files(filename):
        return send_from_directory(os.path.join(app.instance_path, app.config['MEDIA_DIR']),
                                   filename)

    from . import pages
    app.register_blueprint(pages.bp)

    return app
